#! /usr/bin/env python

from __future__ import print_function
import sys, gzip
import pysam

def convert_anno2vcf(input_file, output_file, reference, header = False):

    """
        convert annovar format to vcf format
        the difficulty is: for the annovar format, the reference base for insertion or deletion is removed,
        and we have to recover them...
    """

    hin = open(input_file, 'r')
    hout = open(output_file, 'w')

    """
    # get current date
    today = datetime.date.today()

    # print meta-information lines
    print >> hout, "##fileformat=#VFCv4.1"
    print >> hout, "##fileDate=" + today.strftime("%Y%m%d")
    # print "##source=ebcallv2.0"
    # print "##INFO=<ID=TD,Number=1,Type=Integer,Description=\"Tumor Depth\">"
    # print "##INFO=<ID=TV,Number=1,Type=Integer,Description=\"Tumor Variant Read Num\">"
    # print "##INFO=<ID=ND,Number=1,Type=Integer,Description=\"Normal Depth\">"
    # print "##INFO=<ID=NV,Number=1,Type=Integer,Description=\"Normal Variant Read Num\">"
    print >> hout, "##INFO=<ID=SOMATIC,Number=0,Type=Flag,Description=\"Somatic Variation\">"
    """

    if header == True:
        print("##fileformat=VCFv4.1", file = hout)
        print("#CHROM\tPOS\tID\tREF\tALT\tQUAL\tFILTER\tINFO", file = hout)

    for line in hin:
        F = line.rstrip('\n').split('\t')
        if F[0].startswith('#'): continue
        if F[0] == "Chr" and F[1] == "Start" and F[2] == "End" and F[3] == "Ref" and F[4] == "Alt": continue

        pos, ref, alt = F[1], F[3], F[4]
        
        # insertion
        if F[3] == "-":
            # get the sequence for the reference base
            seq = ""    
            for item in pysam.faidx(reference, F[0] + ":" + str(F[1]) + "-" + str(F[1])):
                seq = seq + item.rstrip('\n')
            seq = seq.replace('>', '')
            seq = seq.replace(F[0] + ":" + str(F[1]) + "-" + str(F[1]), '')
            ref, alt = seq, seq + F[4]


        # deletion
        if F[4] == "-":
            # get the sequence for the reference base
            seq = ""    
            for item in pysam.faidx(reference, F[0] + ":" + str(int(F[1]) - 1) + "-" + str(int(F[1]) - 1)):
                seq = seq + item.rstrip('\n')
            seq = seq.replace('>', '')
            seq = seq.replace(F[0] + ":" + str(int(F[1]) - 1) + "-" + str(int(F[1]) - 1), '')
            pos, ref, alt = str(int(F[1]) - 1), seq + F[3], seq


        # QUAL = int(float(F[15]) * 10)
        QUAL = 60
        # INFO = "TD=" + F[9] + ";TV=" + F[10] + ";ND=" + F[13] + ";NV=" + F[14] + ";SOMATIC"
        INFO = "SOMATIC"

        print(F[0] + "\t" + pos + "\t.\t" + ref + "\t" + alt \
                + "\t" + str(QUAL) + "\t" + "PASS" + "\t" + INFO, file = hout)


    hin.close()
    hout.close()


def remove_vcf_header(input_file, output_file):

    hout = open(output_file, 'w')
    with open(input_file, 'r') as hin:
        for line in hin:
            line = line.rstrip('\n')
            if not line.startswith('#'): print(line, file = hout)

    hout.close()



def convert_genosv2bed(input_file, output_file):

    hout = open(output_file, 'w')
    num = 1
    with open(input_file, 'r') as hin:
        for line in hin:
            F = line.rstrip('\n').split('\t')
            if F[0].startswith('#'): continue
            if F[0] == "Chr_1" and F[1] == "Pos_1": continue
            chr1, pos1, dir1, chr2, pos2, dir2, inseq, sv_type = F[0], F[1], F[2], F[3], F[4], F[5], F[6], F[7]

            if chr1 != chr2: continue
            print('\t'.join([chr1, str(int(pos1) - 1), pos2, chr1, pos1, dir1, chr2, pos2, dir2, inseq, sv_type]), file = hout) 
            # start1, end1 = str(int(F[1]) - 1), F[1]
            # start2, end2 = str(int(F[4]) - 1), F[4]
            # dir1, dir2 = F[2], F[5]
            # name = "SV_" + str(num)
            # inseq = F[6] 

            # print >> hout, '\t'.join([chr1, start1, end1, chr2, start2, end2, name, inseq, dir1, dir2])

    hout.close()


def proc_star_junction(input_file, output_file, control_file, read_num_thres, overhang_thres, remove_annotated, convert_map_splice2):
    
    is_control = True if control_file is not None else False
    # if is_control: control_db = pysam.TabixFile(control_file)
    
    control_db = {}
    if is_control:
        with gzip.open(control_file, 'rt') as hin:
            for line in hin:
                F = line.rstrip('\n').split('\t')
                key = F[0] + '\t' + F[1] + '\t' + F[2]
                control_db[key] = 1


    if read_num_thres is None: read_num_thres = 0
    if overhang_thres is None: overhang_thres = 0
    if remove_annotated is None: remove_annotated = False
    
    hout = open(output_file, 'w')
    with open(input_file, 'r') as hin:
        for line in hin:
            F = line.rstrip('\n').split('\t')
            key = F[0] + '\t' + F[1] + '\t' + F[2]
            if remove_annotated == True and F[5] != "0": continue
            if int(F[6]) < read_num_thres: continue
            if int(F[8]) < overhang_thres: continue


            if key in control_db: continue
            """
            ##########
            # remove control files
            if is_control:
                tabixErrorFlag = 0
                try:
                    records = control_db.fetch(F[0], int(F[1]) - 5, int(F[1]) + 5)
                except Exception as inst:
                    # print >> sys.stderr, "%s: %s" % (type(inst), inst.args)
                    # tabixErrorMsg = str(inst.args)
                    tabixErrorFlag = 1

                control_flag = 0;
                if tabixErrorFlag == 0:
                    for record_line in records:
                        record = record_line.split('\t')
                        if F[0] == record[0] and F[1] == record[1] and F[2] == record[2]:
                            control_flag = 1

                if control_flag == 1: continue
            ##########
            """

            if convert_map_splice2:
                # convert to map-splice2 coordinate
                F[1] = str(int(F[1]) - 1)
                F[2] = str(int(F[2]) + 1)

            print('\t'.join(F), file = hout)

    hout.close()




#! /usr/bin/env python

from .run import *
import argparse
from .version import __version__

def create_parser():

    parser = argparse.ArgumentParser(prog = "junc_utils")

    parser.add_argument("--version", action = "version", version = "%(prog)s " + __version__)

    subparsers = parser.add_subparsers()

    ##########
    # filter

    filter = subparsers.add_parser("filter",
                                   help = "Filter out splicing junctions outside specified conditions")

    filter.add_argument("junc_file", metavar = "sample.SJ.out.tab", default = None, type = str,
                        help = "Path to star junction file")

    filter.add_argument("output_path", metavar = "output.txt", default = None, type = str,
                        help = "Path to the output file")

    filter.add_argument("--read_num_thres", type = int, default = 3,
                        help = "Remove splicing junctions whose supporting numbers are below this value (default: %(default)s)")
                               
    filter.add_argument("--overhang_thres", type = int, default = 10,
                        help = "Remove splicing junctions whose overhang sizes are this value (default: %(default)s)")
                               
    filter.add_argument("--keep_annotated", default = False, action = 'store_true',
                        help = "Do not remove annotated splicing junctions")

    filter.add_argument("--pooled_control_file", default = None, type = str,
                        help = "Path to control data created by merge_control (default: %(default)s)")

    filter.set_defaults(func = filter_main)

    ##########
    # annotate

    annotate = subparsers.add_parser("annotate",
                                     help = "Annotate junctions")

    annotate.add_argument("junc_file", metavar = "sample.SJ.out.tab", default = None, type = str,
                          help = "Path to star splicing junction file")

    annotate.add_argument("output_path", metavar = "output.txt", default = None, type = str,
                          help = "Path to the output file")

    # annotate.add_argument("annotation_dir", metavar = "annotation_dir", default = None, type = str,
    #                       help = "the path to the database directory")

    annotate.add_argument("--gene_model", choices = ["refseq", "gencode"], default = "refseq",
                       help = "gene model (refGene or ensGene) (default: %(default)s)")

    annotate.add_argument("--grc", default = False, action = 'store_true',
                          help = "Deprecated. This is not used any more. Convert chromosome names to Genome Reference Consortium nomenclature (default: %(default)s)")

    annotate.add_argument("--genome_id", choices = ["hg19", "hg38", "mm10"], default = "hg19",
                          help = "Genome id used for selecting UCSC-GRC chromosome name corresponding files (default: %(default)s)")

    annotate.add_argument("--junction_margin", type = int, default = 3,
                          help = "Allowed margin size for the difference between splicing breakpoints and exon-intron junctions")

    annotate.add_argument("--exon_margin", type = int, default = 30,
                          help = "Allowed margin size for the difference between splicing breakpoints and exonic region")

    annotate.set_defaults(func = annotate_main)

    ##########
    # associate

    associate = subparsers.add_parser("associate",
                                     help = "Associate junctions with mutations or SVs")

    associate.add_argument("annotated_junction_file", metavar = "annotated_junction.SJ.out.tab", default = None, type = str,
                           help = "Path to the annotated splicing junction file")

    associate.add_argument("mutation_file", metavar = "mutation.vcf.gz", default = None, type = str,
                           help = "Path to the mutation file")

    associate.add_argument("output_file", metavar = "output_file", default = None, type = str, 
                           help = "Path to the output")

    # associate.add_argument("annotation_dir", metavar = "annotation_dir", type = str,
    #                        help = "the path to database directory")

    associate.add_argument("--grc", default = False, action = 'store_true',
                           help = "Deprecated. This is not used any more. Convert chromosome names to Genome Reference Consortium nomenclature (default: %(default)s)")

    associate.add_argument("--genome_id", choices = ["hg19", "hg38", "mm10"], default = "hg19",
                           help = "Genome id used for selecting UCSC-GRC chromosome name corresponding files (default: %(default)s)")

    associate.add_argument("--donor_size", metavar = "donor_size", default = "3,6", type = str,
                           help = "Splicing donor site size (exonic region size, intronic region size) (default: %(default)s)")

    associate.add_argument("--acceptor_size", metavar = "acceptor_size", default = "6,1", type = str,
                           help = "Splicing donor site size (intronic region size, exonic region size) (default: %(default)s)")

    associate.add_argument("--reference", metavar = "reference.fa", type = str,
                           help = "Path to the reference genomoe sequence (necessary for anno format)")

    associate.add_argument("--debug", default = False, action = 'store_true', help = "Keep intermediate files")

    associate.add_argument("--mutation_format", choices=["vcf", "anno"], default = "vcf",
                           help = "Deprecated. This is not used any more. The format of mutation file vcf or annovar (tsv) format")

    # associate.add_argument("--is_edit_dist", default = False, action = 'store_true', 
    #                        help = "for SNV, only extract those changing edit distance to splicing motifs in consistent manners")

    # associate.add_argument("--skip_creation_indel", default = False, action = 'store_true',
    #                        help = "do not consider indels around splicing junctions (putative splicing motif creatin indels)")

    associate.add_argument('--only_dist', action='store_true',
                           help = "Associate all mutations within the junctions plus specified margin (this is mainly for evaluation)")

    associate.add_argument("--only_dist_search_margin", metavar = "only_dist_search_margin", default = 100, type = int,
                           help = "Margin size used when only_dist option is active (default: %(default)s)")

    associate.add_argument('--sv', action='store_true',
                           help = "Analysis structural variation file")

    associate.add_argument('--branchpoint', action='store_true',
                           help = "Include branch points to the association analysis")

    associate.add_argument("--branchpoint_size", metavar = "branchpoint_size", default = "3,2", type = str,
                           help = "Splicing branchpoint size (intronic-side region size, exonic-side region size) (default: %(default)s)")

    associate.set_defaults(func = associate_main)

    ##########
    # merge control
    merge_control = subparsers.add_parser("merge_control",
                                          help = "Merge, compress and index the splicing junction list (.SJ.out.tab file generated by STAR)")

    merge_control.add_argument("junc_list", metavar = "junc_list.txt", default = None, type = str,
                               help = "Junction path list")

    merge_control.add_argument("output_path", default = None, type = str,
                               help = "Path of the output file")


    merge_control.add_argument("--read_num_thres", type = int, default = 3,
                               help = "Remove splicing junctions whose supporting numbers are below this value (default: %(default)s)")

    merge_control.add_argument("--overhang_thres", type = int, default = 10,
                               help = "Remove splicing junctions whose overhang sizes are this value (default: %(default)s)")

    merge_control.add_argument("--keep_annotated", default = False, action = 'store_true', 
                               help = "Do not remove annotated splicing junctions")

    merge_control.add_argument("--sample_num_thres", type = int, default = 2,
                               help = "Register splicing junctions at least shared by specified number of samples (default: %(default)s)")

    merge_control.set_defaults(func = merge_control_main)

    """
    # temporary remove this function, but want to recover in near future (2018/02/02)
    ##########
    # exonization_pair
    exonization_pair = subparsers.add_parser("exonization_pair",
                                          help = "get candidates for the other junction of an exonization supporting junction pair")

    exonization_pair.add_argument("half_exonizaiton_junction", metavar = "half_exonizaiton_junction", default = None, type = str,
                                  help = "a half of exonization junction pairs")

    exonization_pair.add_argument("junc_file", metavar = "sample.SJ.out.tab", default = None, type = str,
                                  help = "the path to star splicing junction file")

    exonization_pair.add_argument("output_file", metavar = "output_file", default = None, type = str,
                                  help = "the path to the output")

    exonization_pair.add_argument("--control_file", default = None, type = str,
                                  help = "the path to control data created by junc_utils merge_control (default: %(default)s)")

    exonization_pair.add_argument("--grc", default = False, action = 'store_true',
                                  help = "convert chromosome names to Genome Reference Consortium nomenclature (default: %(default)s)")

    exonization_pair.add_argument("--genome_id", choices = ["hg19", "hg38", "mm10"], default = "hg19",
                                  help = "the genome id used for selecting UCSC-GRC chromosome name corresponding files (default: %(default)s)")

    exonization_pair.add_argument("--read_num_thres", type = int, default = 2,
                                  help = "remove splicing junctions whose supporting numbers are below this value (default: %(default)s)")

    exonization_pair.add_argument("--min_new_intron_size", type = int, default = 30,
                                  help = "minimum new intron size")

    exonization_pair.add_argument("--max_new_exon_size", type = int, default = 500,
                                  help = "maximum new exon size")

    exonization_pair.add_argument("--boundary_margin", type = int, default = 3,
                                  help = "margin size when comparing splicing junctions with exon-intron boundaries")

    exonization_pair.set_defaults(func = exonization_pair_main)
    ##########
    """

    return parser




#! /usr/bin/env python

from __future__ import print_function
import sys, os, subprocess
from . import utils
from .logger import get_logger
logger = get_logger(__name__)


def filter_main(args):

    utils.proc_star_junction(args.junc_file, args.output_path, args.pooled_control_file,
                             args.read_num_thres, args.overhang_thres, not args.keep_annotated, False)


def annotate_main(args):
   
    from . import annotate
    from annot_utils.utils import grc_check
    
    if args.grc == True:
        logger.warning("--grc argument is deprecated and ignored.")

    is_grc = grc_check(args.junc_file, [0])
 
    annotate.annot_junction(args.junc_file, args.output_path, args.junction_margin, args.exon_margin, 
                            args.gene_model, args.genome_id, is_grc)


def associate_main(args):

    from . import associate
    from annot_utils.utils import grc_check

    mutation_file = args.mutation_file
    output_file = args.output_file
    # is_anno = True if args.mutation_format == "anno" else False
    is_anno = False if mutation_file.endswith(".vcf") or mutation_file.endswith(".vcf.gz") else True 
    # control_file = args.ctrl
    is_sv = True if args.sv else False
    is_debug = True if args.debug else False 
    reference_genome = args.reference

    if args.mutation_format == "anno":
        logger.warning("--mutation_format is deprepaced and ignored.")

    if args.grc == True:
        logger.warning("--grc argument is deprecated and ignored.")

    is_grc = grc_check(args.annotated_junction_file, [0])
    is_grc_mut = grc_check(mutation_file, [0]) if is_sv == False else grc_check(mutation_file, [0, 3])

    if is_grc != is_grc_mut:
        logger.warning("Splicing junction file and mutation file seems to use different coordinate system.")


    if not is_sv and is_anno and reference_genome is None:
        print("When the mutation file format is annovar format, reference genome should be specified", file = sys.stderr)
        sys.exit(1)

    output_dir = os.path.dirname(output_file)
    if output_dir != "" and not os.path.exists(output_dir):
       os.makedirs(output_dir)

    ##########
    # processing mutation file

    if not is_sv: 

        # convert mutation data to vcf (if --anno is on)
        if is_anno == True:
            utils.convert_anno2vcf(mutation_file, output_file + ".mutran_tmp.unsorted.vcf", reference_genome)
        else:
            utils.remove_vcf_header(mutation_file, output_file + ".mutran_tmp.unsorted.vcf")

        hout = open(output_file + ".mutran_tmp.vcf", 'w')
        s_ret = subprocess.call(["sort", "-f",  "-k1,1", "-k2,2n", output_file + ".mutran_tmp.unsorted.vcf"], stdout = hout)
        hout.close()

        if s_ret != 0:
            print("Error in sorting vcf file", file = sys.stderr)
            sys.exit(1)

        s_ret = subprocess.call(["bgzip", "-f", output_file + ".mutran_tmp.vcf"])
        if s_ret != 0:
            print("Error in bgzip compression", file = sys.stderr)
            sys.exit(1)

        s_ret = subprocess.call(["tabix", "-p", "vcf", output_file + ".mutran_tmp.vcf.gz"])
        if s_ret != 0:
            print("Error in tabix indexing", file = sys.stderr)
            sys.exit(1)

    else:
        
        utils.convert_genosv2bed(mutation_file, output_file + ".mutran_tmp.unsorted.bedpe")

        hout = open(output_file + ".mutran_tmp.bedpe", 'w') 
        s_ret = subprocess.call(["sort", "-f", "-k1,1", "-k2,2n", "-k3,3n", "-k4,4", "-k5,5n", "-k6,6n", output_file + ".mutran_tmp.unsorted.bedpe"], stdout = hout)
        hout.close()
        
        if s_ret != 0:
            print("Error in sorting bedpe file", file = sys.stderr)
            sys.exit(1)
    
        s_ret = subprocess.call(["bgzip", "-f", output_file + ".mutran_tmp.bedpe"])
        if s_ret != 0:
            print("Error in bgzip compression", file = sys.stderr)
            sys.exit(1)
    
        
        s_ret = subprocess.call(["tabix", "-p", "bed", output_file + ".mutran_tmp.bedpe.gz"])
        if s_ret != 0:
            print("Error in tabix indexing", file = sys.stderr)
            sys.exit(1)

    ##########

    ##########
    # associate mutation and junction
    if not is_sv:
        """
        associate.get_snv_junction(args.annotated_junction_file,
                                   output_file,
                                   output_file + ".mutran_tmp.vcf.gz",
                                   args.annotation_dir, True, args.skip_creation_indel)
                                   # args.annotation_dir, args.is_edit_dist, args.skip_creation_indel)
        """

        if not args.only_dist:
            associate.get_snv_junction2(args.annotated_junction_file,
                                        output_file,
                                        output_file + ".mutran_tmp.vcf.gz",
                                        args.donor_size, args.acceptor_size,
                                        args.genome_id, is_grc,
                                        args.branchpoint, args.branchpoint_size)
        else:
            associate.get_snv_junction_only_dist(args.annotated_junction_file, 
                                                 output_file, 
                                                 output_file + ".mutran_tmp.vcf.gz",
                                                 args.annotation_dir, args.only_dist_search_margin)

    else:
        associate.get_sv_junction(args.annotated_junction_file,
                                  output_file,
                                  output_file + ".mutran_tmp.bedpe.gz",               
                                  args.genome_id, is_grc)
    ##########

    if is_debug != True:
        if not is_sv:
            subprocess.call(["rm", "-rf", output_file + ".mutran_tmp.unsorted.vcf"])
            subprocess.call(["rm", "-rf", output_file + ".mutran_tmp.vcf.gz"])
            subprocess.call(["rm", "-rf", output_file + ".mutran_tmp.vcf.gz.tbi"])
        else:
            subprocess.call(["rm", "-rf", output_file + ".mutran_tmp.unsorted.bedpe"])
            subprocess.call(["rm", "-rf", output_file + ".mutran_tmp.bedpe.gz"])
            subprocess.call(["rm", "-rf", output_file + ".mutran_tmp.bedpe.gz.tbi"])

        subprocess.call(["rm", "-rf", output_file + ".mutran_tmp.junction.txt"])
        subprocess.call(["rm", "-rf", output_file + ".mutran_tmp.junction.annot.txt"])



def merge_control_main(args):

    input_file_list = args.junc_list
    output_file = args.output_path
    read_num_thres = args.read_num_thres
    overhang_thres = args.overhang_thres
    remove_annotated = False if args.keep_annotated else True
    sample_num_thres = args.sample_num_thres

    # make directory for output if necessary
    if os.path.dirname(output_file) != "" and not os.path.exists(os.path.dirname(output_file)):
        os.makedirs(os.path.dirname(output_file))

    # hin = open(input_file_list, 'r')
    hout = open(output_file + ".unsorted", 'w')
    with open(input_file_list, 'r') as hin:
        for line in hin:

            junction_file = line.rstrip('\n')
            with open(junction_file, 'r') as hin2:
                for line2 in hin2:

                    F = line2.rstrip('\n').split('\t')
                    if remove_annotated == True and F[5] != "0": continue
                    if int(F[6]) < read_num_thres: continue 
                    if int(F[8]) < overhang_thres: continue 
         
                    # convert to map-splice2 coordinate
                    # F[1] = str(int(F[1]) - 1)
                    # F[2] = str(int(F[2]) + 1)
         
                    print('\t'.join(F), file = hout)

    hout.close()                


    hout = open(output_file + ".sorted", 'w')
    s_ret = subprocess.call(["sort", "-f", "-k1,1", "-k2,2n", "-k3,3n", output_file + ".unsorted"], stdout = hout)
    hout.close()

    if s_ret != 0:
        print("Error in sorting merged junction file", file = sys.stderr)
        sys.exit(1)


    hout = open(output_file + ".merged", 'w')
    with open(output_file + ".sorted", 'r') as hin:
        temp_key = ""
        temp_read_num = []
        for line in hin:
            F = line.rstrip('\n').split('\t')
            key = F[0] + '\t' + F[1] + '\t' + F[2]
            read_num = int(F[6])
            if key != temp_key:
                if temp_key != "":
                    if len(temp_read_num) >= sample_num_thres:
                        print(temp_key + '\t' + ','.join([str(x) for x in sorted(temp_read_num)]), file = hout)
                temp_key = key
                temp_read_num = []

            temp_read_num.append(read_num)

        if temp_key != "":
            if len(temp_read_num) >= sample_num_thres:
                print(temp_key + '\t' + ','.join([str(x) for x in sorted(temp_read_num)]), file = hout)

    hout.close()


    hout = open(output_file, 'w')
    s_ret = subprocess.call(["bgzip", "-f", "-c", output_file + ".merged"], stdout = hout)
    hout.close()

    if s_ret != 0:
        print("Error in compression merged junction file", file = sys.stderr)
        sys.exit(1)


    s_ret = subprocess.call(["tabix", "-p", "bed", output_file])
    if s_ret != 0:
        print("Error in indexing merged junction file", file = sys.stderr)
        sys.exit(1)

    subprocess.call(["rm", "-f", output_file + ".unsorted"])
    subprocess.call(["rm", "-f", output_file + ".sorted"])
    subprocess.call(["rm", "-f", output_file + ".merged"])



def exonization_pair_main(args):

    # investigate the input splicing junction
    # 1. whether the both ends are located within the same gene
    # 2. whether one of the ends is at the authentic splicing junction
    # 3. whether one of the ends is located on intronic region
    # 4. the opposite authentic splicing junction to the one of the ends of input splicing junction

    # exonization_info = [chromosome, authentic exon-intron boundary, opposite exon-intron boundary, new splice site, strand, donor or acceptor]

    import exonization_pair

    exonization_info = exonization_pair.check_splicing_junction_for_exonization(args.half_exonizaiton_junction, args.output_file, 
                         args.genome_id, args.grc, args.min_new_intron_size, args.boundary_margin)
    
    # print exonization_info

    if len(exonization_info) == 0: 
        open(args.output_file, 'a').close()
    else:
        exonization_pair.check_opposite_junction(args.junc_file, exonization_info, args.output_file, args.control_file,
                                                 args.read_num_thres, args.max_new_exon_size, args.boundary_margin)

 




#! /usr/bin/env python

from __future__ import print_function
import pysam 
import sys, subprocess

# for python2 and 3 compatibility
try:
    import itertools.izip as zip
except ImportError:
    pass

import annot_utils.gene, annot_utils.exon, annot_utils.coding


def annot_junction(input_file, output_file, junction_margin, exon_margin, gene_model = "refseq", genome_id = "hg19", is_grc = True):
    """
        The purpose of this script is to classify splicing changes
        mainly by comparing the two breakpoints with the exon-intorn junction of genes
        within the database.
        Also, we generate the sequence arrond the breakpoints, which will be helpful
        for checking the authenticity of the splicing and evaluating the relationships
        with the somatic mutations.

        here is the classification categories:
        1. known (The splicing pattern is included in the database)
        (the start and end breakpoints are next exon-intron junctions of the same gene) 
        2. exon skipping 
        (the start and end breakpoints are exon-intron junctions of the same gene,
         but not the next ones)
        3. splice-site slip
        (one of the two breakpoints is an exon-intron junction and the other is within the 30bp exon of the same gene)
        4. pseudo-exon inclusion
        (one of the two break points is an exon-intron junction and the other is located in the same gene, but more than 30bp from exons of the gene)
        5. other
        (neighter of the two breakpoins are exon-intron junction, but located in the same gene)
        6. chimeric (spliced)
        7. chimeric (un-spliced)


        The algorithm for the annotation is as follows
        1. for both breakpoints, list up the exon-intron junctions matching to the breakpoints
        2. for both breakpoints, list up the exons within 30bp from the breakpoints
        3. for both breakpoints, list up the genes matching to the breakpoints
        4. summarize the above results and induce the annotation from them
        5. get the sequence arround the breakpoints.

    """

    annot_utils.gene.make_gene_info(output_file + ".tmp.refGene.bed.gz", gene_model, genome_id, is_grc, True)
    annot_utils.exon.make_exon_info(output_file + ".tmp.refExon.bed.gz", gene_model, genome_id, is_grc, True)
    annot_utils.coding.make_coding_info(output_file + ".tmp.refCoding.bed.gz", gene_model, genome_id, is_grc, True)
  
    with open(output_file + ".tmp1.junc1.gene.bed", 'w') as hout_g1, open(output_file + ".tmp1.junc2.gene.bed", 'w') as hout_g2, \
      open(output_file + ".tmp1.junc1.exon.bed", 'w') as hout_e1, open(output_file + ".tmp1.junc2.exon.bed", 'w') as hout_e2:
        with open(input_file, 'r') as hin:
            for line in hin:
                F = line.rstrip('\n').split('\t')
                chr_name, sj_start, sj_end = F[0], int(F[1]) - 1, int(F[2]) + 1
                sj_id = ','.join([F[0], F[1], F[2]])

                print(f'{chr_name}\t{max(0, sj_start - 1)}\t{sj_start}\t{sj_id}', file = hout_g1)
                print(f'{chr_name}\t{max(0, sj_end - 1)}\t{sj_end}\t{sj_id}', file = hout_g2)
                print(f'{chr_name}\t{max(0, sj_start - exon_margin - 1)}\t{sj_start + exon_margin}\t{sj_id}', file = hout_e1)
                print(f'{chr_name}\t{max(0, sj_end - exon_margin - 1)}\t{sj_end + exon_margin}\t{sj_id}', file = hout_e2) 
                """
                print(chr_name + '\t' + str(sj_start - 1) + '\t' + str(sj_start) + '\t' + sj_id, file = hout_g1)
                print(chr_name + '\t' + str(sj_end - 1) + '\t' + str(sj_end) + '\t' + sj_id, file = hout_g2)
                print(chr_name + '\t' + str(sj_start - exon_margin - 1) + '\t' + str(sj_start + exon_margin) + '\t' + sj_id, file = hout_e1)
                print(chr_name + '\t' + str(sj_end - exon_margin - 1) + '\t' + str(sj_end + exon_margin) + '\t' + sj_id, file = hout_e2)
                """

    with open(output_file + ".tmp2.junc1.gene.bed", 'w') as hout_g1:
        subprocess.check_call(["bedtools", "intersect", "-a", output_file + ".tmp1.junc1.gene.bed", "-b", output_file + ".tmp.refGene.bed.gz", "-loj"], stdout = hout_g1)

    with open(output_file + ".tmp2.junc2.gene.bed", 'w') as hout_g2:
        subprocess.check_call(["bedtools", "intersect", "-a", output_file + ".tmp1.junc2.gene.bed", "-b", output_file + ".tmp.refGene.bed.gz", "-loj"], stdout = hout_g2)

    with open(output_file + ".tmp2.junc1.exon.bed", 'w') as hout_e1:
        subprocess.check_call(["bedtools", "intersect", "-a", output_file + ".tmp1.junc1.exon.bed", "-b", output_file + ".tmp.refExon.bed.gz", "-loj"], stdout = hout_e1)

    with open(output_file + ".tmp2.junc2.exon.bed", 'w') as hout_e2:
        subprocess.check_call(["bedtools", "intersect", "-a", output_file + ".tmp1.junc2.exon.bed", "-b", output_file + ".tmp.refExon.bed.gz", "-loj"], stdout = hout_e2)

    subprocess.check_call(["rm", "-rf", output_file + ".tmp.refGene.bed.gz"])
    subprocess.check_call(["rm", "-rf", output_file + ".tmp.refGene.bed.gz.tbi"])
    subprocess.check_call(["rm", "-rf", output_file + ".tmp.refExon.bed.gz"])
    subprocess.check_call(["rm", "-rf", output_file + ".tmp.refExon.bed.gz.tbi"])
    subprocess.check_call(["rm", "-rf", output_file + ".tmp1.junc1.gene.bed"])
    subprocess.check_call(["rm", "-rf", output_file + ".tmp1.junc2.gene.bed"])
    subprocess.check_call(["rm", "-rf", output_file + ".tmp1.junc1.exon.bed"])
    subprocess.check_call(["rm", "-rf", output_file + ".tmp1.junc2.exon.bed"])

       
    with open(output_file + ".tmp2.junc1.gene.bed", 'r') as hin, open(output_file + ".tmp3.junc1.gene.bed", 'w') as hout:
        tmp_id, tmp_gene = "", []
        for line in hin:
            F = line.rstrip('\n').split('\t')
            if F[3] != tmp_id:
                if tmp_id != "": print(tmp_id + '\t' + ','.join(list(set(tmp_gene))), file = hout)
                tmp_id, tmp_gene = F[3], []
            if F[7] != ".":
                tmp_gene.append(F[7])
            else:
                tmp_gene.append("---")

        if tmp_id != "": print(tmp_id + '\t' + ','.join(list(set(tmp_gene))), file = hout)

    with open(output_file + ".tmp2.junc2.gene.bed", 'r') as hin, open(output_file + ".tmp3.junc2.gene.bed", 'w') as hout:
        tmp_id, tmp_gene = "", []
        for line in hin:
            F = line.rstrip('\n').split('\t')
            if F[3] != tmp_id:
                if tmp_id != "": print(tmp_id + '\t' + ','.join(list(set(tmp_gene))), file = hout)
                tmp_id, tmp_gene = F[3], []
            if F[7] != ".":
                tmp_gene.append(F[7])
            else:
                tmp_gene.append("---")

        if tmp_id != "": print(tmp_id + '\t' + ','.join(list(set(tmp_gene))), file = hout)

    with open(output_file + ".tmp2.junc1.exon.bed", 'r') as hin, open(output_file + ".tmp3.junc1.exon.bed", 'w') as hout:
        tmp_id, tmp_gene, tmp_exon_num, tmp_edge, tmp_offset = "", [], [], [], []
        for line in hin:
            F = line.rstrip('\n').split('\t')
            FF = F[3].split(',')
            chr_name, sj_start, sj_end = FF[0], int(FF[1]) - 1, int(FF[2]) + 1

            if F[3] != tmp_id:
                if tmp_id != "":
                    if not len(tmp_gene) == len(tmp_exon_num) == len(tmp_edge) == len(tmp_offset):
                        print("Inconsistency for the format in creating exon information files", file = sys.stderr)
                        sys.exit(1)
                    print('\t'.join([tmp_id, ','.join(tmp_gene), ','.join(tmp_exon_num), ','.join(tmp_edge), ','.join(tmp_offset)]), file = hout)
                tmp_id, tmp_gene, tmp_exon_num, tmp_edge, tmp_offset = F[3], [], [], [], []

            if F[7] != ".":
                tmp_gene.append(F[7])
                tmp_exon_num.append(F[8])
                if abs(sj_start - int(F[6])) < junction_margin:
                    tmp_offset.append(str(sj_start - int(F[6])))
                    if F[9] == '+': tmp_edge.append('e')
                    if F[9] == '-': tmp_edge.append('s')
                else:
                    tmp_edge.append("---"), tmp_offset.append("---")
            else:
                tmp_gene.append("---"), tmp_exon_num.append("---"), tmp_edge.append("---"), tmp_offset.append("---")

        if tmp_id != "":
            if not len(tmp_gene) == len(tmp_exon_num) == len(tmp_edge) == len(tmp_offset):
                print("Inconsistency for the format in creating exon information files", file = sys.stderr)
                sys.exit(1)
            print('\t'.join([tmp_id, ','.join(tmp_gene), ','.join(tmp_exon_num), ','.join(tmp_edge), ','.join(tmp_offset)]), file = hout)

    with open(output_file + ".tmp2.junc2.exon.bed", 'r') as hin, open(output_file + ".tmp3.junc2.exon.bed", 'w') as hout:
        tmp_id, tmp_gene, tmp_exon_num, tmp_edge, tmp_offset = "", [], [], [], []
        for line in hin:
            F = line.rstrip('\n').split('\t')
            FF = F[3].split(',')
            chr_name, sj_start, sj_end = FF[0], int(FF[1]) - 1, int(FF[2]) + 1

            if F[3] != tmp_id:
                if tmp_id != "":
                    if not len(tmp_gene) == len(tmp_exon_num) == len(tmp_edge) == len(tmp_offset):
                        print("Inconsistency for the format in creating exon information files", file = sys.stderr)
                        sys.exit(1)
                    print('\t'.join([tmp_id, ','.join(tmp_gene), ','.join(tmp_exon_num), ','.join(tmp_edge), ','.join(tmp_offset)]), file = hout)
                tmp_id, tmp_gene, tmp_exon_num, tmp_edge, tmp_offset = F[3], [], [], [], []

            if F[7] != ".":
                tmp_gene.append(F[7])
                tmp_exon_num.append(F[8])
                if abs(sj_end - 1 - int(F[5])) < junction_margin:
                    tmp_offset.append(str(sj_end - 1 - int(F[5])))
                    if F[9] == '+': tmp_edge.append('s')
                    if F[9] == '-': tmp_edge.append('e')
                else:
                    tmp_edge.append("---"), tmp_offset.append("---")
            else:
                tmp_gene.append("---"), tmp_exon_num.append("---"), tmp_edge.append("---"), tmp_offset.append("---")

        if tmp_id != "":
            if not len(tmp_gene) == len(tmp_exon_num) == len(tmp_edge) == len(tmp_offset):
                print("Inconsistency for the format in creating exon information files", file = sys.stderr)
                sys.exit(1)
            print('\t'.join([tmp_id, ','.join(tmp_gene), ','.join(tmp_exon_num), ','.join(tmp_edge), ','.join(tmp_offset)]), file = hout)

    subprocess.check_call(["rm", "-rf", output_file + ".tmp2.junc1.gene.bed"])
    subprocess.check_call(["rm", "-rf", output_file + ".tmp2.junc2.gene.bed"])
    subprocess.check_call(["rm", "-rf", output_file + ".tmp2.junc1.exon.bed"])
    subprocess.check_call(["rm", "-rf", output_file + ".tmp2.junc2.exon.bed"])

    ##########
    # for creating splicing junction to in-frame information table
    with open(output_file + ".tmp3.junc1.gene.bed") as hin_g1, open(output_file + ".tmp3.junc2.gene.bed") as hin_g2, \
      open(output_file + ".tmp3.junc12.gene.bed", 'w') as hout:
        for line_g1, line_g2 in zip(hin_g1, hin_g2):
            F_g1 = line_g1.rstrip('\n').split('\t')
            F_g2 = line_g2.rstrip('\n').split('\t')

            # key check
            if not F_g1[0] == F_g2[0]:
                print("Inconsistency of splicing junction keys in the information files: " + output_file + ".tmp3.junc12.gene.bed", file = sys.stderr)
                print(F_g1[0] + ' ' + F_g2[0])
                sys.exit(1)

            FF = F_g1[0].split(',')
            chr_name, sj_start, sj_end = FF[0], int(FF[1]) - 1, int(FF[2]) + 1

            gene1 = F_g1[1].split(',') if F_g1[1] != "---" else []
            gene2 = F_g2[1].split(',') if F_g2[1] != "---" else []

            common_gene_list = list(set(gene1) & set(gene2))
            common_gene = ','.join(common_gene_list) if len(common_gene_list) > 0 else "---"

            print(f'{chr_name}\t{max(0, sj_start - exon_margin - 1)}\t{sj_end + exon_margin}\t{F_g1[0]}\t{common_gene}', file = hout)

            """
            print(chr_name + '\t' + str(sj_start - exon_margin - 1) + '\t' + str(sj_end + exon_margin) + '\t' + F_g1[0] + '\t' + common_gene, file = hout)
            """

    with open(output_file + ".tmp3.junc12.gene.coding.bed", 'w') as hout:
        subprocess.check_call(["bedtools", "intersect", "-a", output_file + ".tmp3.junc12.gene.bed", "-b", output_file + ".tmp.refCoding.bed.gz", "-loj"], stdout = hout)

    with open(output_file + ".tmp3.junc12.gene.coding.bed") as hin, open(output_file + ".tmp3.junc12.coding_size.bed", 'w') as hout:
        tmp_id, tmp_sj_start, tmp_sj_end, tmp_gene2coding_size = "", 0, 0, {}
        for line in hin:
            F = line.rstrip('\n').split('\t')
            if tmp_id != F[3]:
                if tmp_id != "":
                    if len(tmp_gene2coding_size) == 0: tmp_gene2coding_size["---"] = "---"
                    print(tmp_id + '\t' + ','.join(tmp_gene2coding_size.keys()) + '\t' + \
                      ','.join([str(x) for x in tmp_gene2coding_size.values()]), file = hout)

                tmp_id = F[3]
                _, tmp_sj_start, tmp_sj_end = tmp_id.split(',')
                tmp_sj_start, tmp_sj_end = int(tmp_sj_start) - 1, int(tmp_sj_end) + 1 
                tmp_gene2coding_size = {}

            if F[4] == "---": continue
            if F[9] != "coding": continue
            if F[8] not in F[4].split(','): continue

            exon_start, exon_end, exon_gene = int(F[6]), int(F[7]), F[8]
            if exon_gene not in tmp_gene2coding_size: tmp_gene2coding_size[exon_gene] = 0

            # sj_start overlaps with the current coding region
            if exon_start <= tmp_sj_start + exon_margin and tmp_sj_start - exon_margin <= exon_end:
                # sj_end also overlaps with the current coding region
                if exon_start <= tmp_sj_end + exon_margin and tmp_sj_end - exon_margin <= exon_end:
                    tmp_gene2coding_size[exon_gene] = tmp_gene2coding_size[exon_gene] + tmp_sj_end - tmp_sj_start - 1
                else:
                    tmp_gene2coding_size[exon_gene] = tmp_gene2coding_size[exon_gene] + exon_end - tmp_sj_start
            # sj_end overlaps with the current coding region
            elif exon_start <= tmp_sj_end + exon_margin and tmp_sj_end - exon_margin <= exon_end:
                tmp_gene2coding_size[exon_gene] = tmp_gene2coding_size[exon_gene] + tmp_sj_end - 1 - exon_start 
            else:                
                tmp_gene2coding_size[exon_gene] = tmp_gene2coding_size[exon_gene] + exon_end - exon_start

        if tmp_id != "":
            if len(tmp_gene2coding_size) == 0: tmp_gene2coding_size["---"] = "---"
            print(tmp_id + '\t' + ','.join(tmp_gene2coding_size.keys()) + '\t' + \
                  ','.join([str(x) for x in tmp_gene2coding_size.values()]), file = hout)
 
    subprocess.check_call(["rm", "-rf", output_file + ".tmp.refCoding.bed.gz"])
    subprocess.check_call(["rm", "-rf", output_file + ".tmp.refCoding.bed.gz.tbi"])
    subprocess.check_call(["rm", "-rf", output_file + ".tmp3.junc12.gene.bed"])
    subprocess.check_call(["rm", "-rf", output_file + ".tmp3.junc12.gene.coding.bed"])
    ##########


    with open(input_file, 'r') as hin, open(output_file, 'w') as hout:
        line = hin.readline()
        F = line.rstrip('\n').split('\t')
        print('\t'.join(["SJ_" + str(i) for i in range(1, len(F) + 1)]) + '\t' + \
              "Splicing_Class" + '\t' + "Is_Inframe" + '\t' + "Gene_1" + '\t' + "Exon_Num_1" + '\t' + \
              "Is_Boundary_1" + '\t' + "Offset_1" + '\t' + "Gene_2" + '\t' + "Exon_Num_2" + '\t' + \
              "Is_Boundary_2" + '\t' + "Offset_2", file = hout)

    with open(input_file, 'r') as hin, open(output_file + ".tmp3.junc1.gene.bed", 'r') as hin_g1, open(output_file + ".tmp3.junc2.gene.bed", 'r') as hin_g2, \
      open(output_file + ".tmp3.junc1.exon.bed", 'r') as hin_e1, open(output_file + ".tmp3.junc2.exon.bed", 'r') as hin_e2, \
      open(output_file + ".tmp3.junc12.coding_size.bed") as hin_c, open(output_file, 'a') as hout:

        for line, line_g1, line_g2, line_e1, line_e2, line_c in zip(hin, hin_g1, hin_g2, hin_e1, hin_e2, hin_c):

            F = line.rstrip('\n').split('\t')
            F_g1, F_g2, F_e1, F_e2, F_c = line_g1.rstrip('\n').split('\t'), line_g2.rstrip('\n').split('\t'), \
                                            line_e1.rstrip('\n').split('\t'), line_e2.rstrip('\n').split('\t'), line_c.rstrip('\n').split('\t')

            chr_name, sj_start, sj_end = F[0], int(F[1]) - 1, int(F[1]) + 1
            sj_id = ','.join([F[0], F[1], F[2]])
            ##########

            # key check
            if not sj_id == F_g1[0] == F_g2[0] == F_e1[0] == F_e2[0] == F_c[0]:
                print("Inconsistency of splicing junction keys in the information files", file = sys.stderr)
                print(sj_id, F_g1[0], F_g2[0], F_e1[0], F_e2[0], F_c[0])
                sys.exit(1)

            gene1 = F_g1[1].split(',') if F_g1[1] != "---" else []
            gene2 = F_g2[1].split(',') if F_g2[1] != "---" else [] 

            exon1, junction1, offset1 = {}, {}, {}
            for tmp_gene, tmp_exon_num, tmp_edge, tmp_offset in zip(F_e1[1].split(','), F_e1[2].split(','), F_e1[3].split(','), F_e1[4].split(',')):
                if tmp_gene == "---": continue
                exon1[tmp_gene] = int(tmp_exon_num)
                if tmp_edge != "---": junction1[tmp_gene] = tmp_edge
                if tmp_offset != "---": offset1[tmp_gene] = tmp_offset
                 
            exon2, junction2, offset2 = {}, {}, {}
            for tmp_gene, tmp_exon_num, tmp_edge, tmp_offset in zip(F_e2[1].split(','), F_e2[2].split(','), F_e2[3].split(','), F_e2[4].split(',')):
                if tmp_gene == "---": continue
                exon2[tmp_gene] = int(tmp_exon_num)
                if tmp_edge != "---": junction2[tmp_gene] = tmp_edge
                if tmp_offset != "---": offset2[tmp_gene] = tmp_offset


            gene2coding_size = {}
            for tmp_gene, tmp_coding_size in zip(F_c[1].split(','), F_c[2].split(',')):
                if tmp_gene == "---": continue
                gene2coding_size[tmp_gene] = int(tmp_coding_size)
 

            spliceClass = ""
            in_frame = "---"
            checkGenes = list(set(gene1 + gene2))
            ##########
            # check for know junction
            passGene = []
            for gene in checkGenes:
                if gene in gene1 and gene in gene2 and gene in junction1 and gene in junction2:
                    if junction1[gene] == "e" and junction2[gene] == "s" and exon2[gene] - exon1[gene] == 1: passGene.append(gene)
                    if junction2[gene] == "e" and junction1[gene] == "s" and exon1[gene] - exon2[gene] == 1: passGene.append(gene)

            if len(passGene) > 0: spliceClass = "Known"

            ##########
            # check for exon skip
            if spliceClass == "":
                passGene = []
                inframe_gene = []
                for gene in checkGenes:
                    if gene in gene1 and gene in gene2 and gene in junction1 and gene in junction2:
                        if (junction1[gene] == "e" and junction2[gene] == "s" and exon2[gene] - exon1[gene] > 1) or \
                           (junction2[gene] == "e" and junction1[gene] == "s" and exon1[gene] - exon2[gene] > 1): 
                            passGene.append(gene)

                            sc_size = gene2coding_size[gene] if gene in gene2coding_size else 0
                            # sc_size = spliced_coding_size(gene, None, chr_name, sj_start, sj_end, coding_tb, exon_margin)
                            if sc_size != 0 and sc_size % 3 == 0:
                                inframe_gene.append(gene)

                if len(passGene) > 0: spliceClass = "Exon skipping"
                if len(inframe_gene) > 0: in_frame = "In-frame"

            ##########
            # check for alternative-3'-splice-site
            if spliceClass == "":
                passGene = []
                inframe_gene = []
                for gene in checkGenes:
                    if gene in gene1 and gene in gene2:
                        if (gene in junction1 and junction1[gene] == "e" and gene not in junction2 and gene in exon2) or \
                           (gene in junction2 and junction2[gene] == "e" and gene not in junction1 and gene in exon1):
                            passGene.append(gene)

                            sc_size = gene2coding_size[gene] if gene in gene2coding_size else 0
                            # sc_size = spliced_coding_size(gene, None, chr_name, sj_start, sj_end, coding_tb, exon_margin)
                            if sc_size != 0 and sc_size % 3 == 0:
                                inframe_gene.append(gene)

                if len(passGene) > 0: spliceClass = "Alternative 3'SS"
                if len(inframe_gene) > 0: in_frame = "In-frame"


            # check for alternative-5'-splice-site
            if spliceClass == "":
                passGene = []
                inframe_gene = []
                for gene in checkGenes:
                    if gene in gene1 and gene in gene2:
                        if (gene in junction1 and junction1[gene] == "s" and gene not in junction2 and gene in exon2) or \
                           (gene in junction2 and junction2[gene] == "s" and gene not in junction1 and gene in exon1):
                            passGene.append(gene)

                            sc_size = gene2coding_size[gene] if gene in gene2coding_size else 0
                            # sc_size = spliced_coding_size(gene, None, chr_name, sj_start, sj_end, coding_tb, exon_margin)
                            if sc_size != 0 and sc_size % 3 == 0:
                                inframe_gene.append(gene)

                if len(passGene) > 0: spliceClass = "Alternative 5'SS"
                if len(inframe_gene) > 0: in_frame = "In-frame"

            ##########
            # check for intronic-alternative-3'-splice-site
            if spliceClass == "":
                passGene = []
                inframe_gene = []
                for gene in checkGenes:
                    if gene in gene1 and gene in gene2:
                        if (gene in junction1 and junction1[gene] == "e" and gene not in junction2 and gene not in exon2) or \
                           (gene in junction2 and junction2[gene] == "e" and gene not in junction1 and gene not in exon1):
                            passGene.append(gene)
            
                if len(passGene) > 0: spliceClass = "Intronic alternative 3'SS"

            # check for intronic-alternative-5'-splice-site
            if spliceClass == "":
                passGene = []
                inframe_gene = []
                for gene in checkGenes:
                    if gene in gene1 and gene in gene2:
                        if (gene in junction1 and junction1[gene] == "s" and gene not in junction2 and gene not in exon2) or \
                           (gene in junction2 and junction2[gene] == "s" and gene not in junction1 and gene not in exon1):
                            passGene.append(gene)
                
                if len(passGene) > 0: spliceClass = "Intronic alternative 5'SS"

            ##########
            # within-exon
            if spliceClass == "":
                passGene = []
                inframe_gene = []
                for gene in checkGenes:
                    if gene in gene1 and gene in gene2 and gene in exon1 and gene in exon2:
                        if exon1[gene] == exon2[gene]:
                            passGene.append(gene)

                            sc_size = gene2coding_size[gene] if gene in gene2coding_size else 0
                            # sc_size = spliced_coding_size(gene, None, chr_name, sj_start, sj_end, coding_tb, exon_margin)
                            if sc_size != 0 and sc_size % 3 == 0:
                                inframe_gene.append(gene)

                if len(passGene) > 0: spliceClass = "Within exon"
                if len(inframe_gene) > 0: in_frame = "In-frame"

            ##########
            # check for exon-exon-junction
            if spliceClass == "":
                passGene = []
                inframe_gene = []
                for gene in checkGenes:
                    if gene in gene1 and gene in gene2:
                        if gene in exon1 and gene in exon2:
                            passGene.append(gene)

                            sc_size = gene2coding_size[gene] if gene in gene2coding_size else 0
                            # sc_size = spliced_coding_size(gene, None, chr_name, sj_start, sj_end, coding_tb, exon_margin)
                            if sc_size != 0 and sc_size % 3 == 0:
                                inframe_gene.append(gene)

                if len(passGene) > 0: spliceClass = "Exon exon junction"
                if len(inframe_gene) > 0: in_frame = "In-frame"

            ##########
            # check for within-gene 
            if spliceClass == "":
                passGene = []
                for gene in checkGenes:
                    if gene in gene1 and gene in gene2 and gene: passGene.append(gene)   
            
                if len(passGene) > 0: spliceClass = "Within gene"


            ##########
            # check for spliced-chimera 
            if spliceClass == "":
                passGene = []
                for g1 in gene1:
                    for g2 in gene2:
                        if (g1 in junction1 and junction1[g1] == "s" and g2 in junction2 and junction2[g2] == "e") or \
                           (g1 in junction1 and junction1[g1] == "e" and g2 in junction2 and junction2[g2] == "s"): 
                            passGene.append(g1 + ',' + g2)

                            # sc_size = gene2coding_size[gene] if gene in gene2coding_size else 0
                            # sc_size = spliced_coding_size(gene, None, chr_name, sj_start, sj_end, coding_tb, exon_margin)
                            # if sc_size != 0 and sc_size % 3 == 0:
                            #     inframe_gene.append(gene)


                if len(passGene) > 0: spliceClass = "Spliced chimera"
                # if len(inframe_gene) > 0: in_frame = "in-frame"

            ##########
            # check for unspliced-chimera 
            if spliceClass == "":
                passGene = []
                for g1 in gene1:
                    for g2 in gene2:
                        passGene.append(g1 + ',' + g2)

                if len(passGene) > 0: spliceClass = "Unspliced chimera"


            if spliceClass == "": spliceClass = "Other"
            

            # summarize the exon and junction information for display
            geneInfo1 = []
            exonInfo1 = []
            junctionInfo1 = []
            offsetInfo1 = []
            if len(gene1) > 0:
                for g1 in sorted(gene1):
                    if g1 not in passGene: continue 
                    geneInfo1.append(g1)
                    if g1 in exon1: 
                        exonInfo1.append(str(exon1[g1]))
                    else:
                        exonInfo1.append("*")

                    if g1 in junction1:
                        junctionInfo1.append(junction1[g1])
                    else:
                        junctionInfo1.append("*")

                    if g1 in offset1:
                        offsetInfo1.append(str(offset1[g1]))
                    else:
                        offsetInfo1.append("*")

            if len(geneInfo1) == 0: 
                geneInfo1.append("---")
                exonInfo1.append("---")
                junctionInfo1.append("---")
                offsetInfo1.append("---")

            geneInfo2 = []
            exonInfo2 = []
            junctionInfo2 = []
            offsetInfo2 = []
            if len(gene2) > 0:
                for g2 in sorted(gene2):
                    if g2 not in passGene: continue
                    geneInfo2.append(g2)
                    if g2 in exon2:
                        exonInfo2.append(str(exon2[g2]))
                    else:
                        exonInfo2.append("*")
                    
                    if g2 in junction2:
                        junctionInfo2.append(junction2[g2])
                    else:
                        junctionInfo2.append("*")

                    if g2 in offset2:
                        offsetInfo2.append(str(offset2[g2]))
                    else:
                        offsetInfo2.append("*")

            if len(geneInfo2) == 0:
                geneInfo2.append("---")
                exonInfo2.append("---")
                junctionInfo2.append("---")
                offsetInfo2.append("---")

         

            print('\t'.join(F) + '\t' + spliceClass + '\t' + in_frame + '\t' + '\t'.join([';'.join(geneInfo1), ';'.join(exonInfo1), ';'.join(junctionInfo1), ';'.join(offsetInfo1), ';'.join(geneInfo2), ';'.join(exonInfo2), ';'.join(junctionInfo2), ';'.join(offsetInfo2)]), file = hout)
     
    subprocess.check_call(["rm", "-rf", output_file + ".tmp3.junc1.gene.bed"])
    subprocess.check_call(["rm", "-rf", output_file + ".tmp3.junc2.gene.bed"])
    subprocess.check_call(["rm", "-rf", output_file + ".tmp3.junc1.exon.bed"])
    subprocess.check_call(["rm", "-rf", output_file + ".tmp3.junc2.exon.bed"])
    subprocess.check_call(["rm", "-rf", output_file + ".tmp3.junc12.coding_size.bed"])



#! /usr/bin/env python

from __future__ import print_function
import sys, subprocess, pkg_resources
import pysam
import annot_utils.exon


def get_snv_junction2(input_file, output_file, mutation_file, donor_size, acceptor_size, 
                      genome_id, is_grc, is_branchpoint, branch_size):

    """
        a script for detecting candidate somatic substitutions causing splicing changes
        
        1. exon-skip:
           exon-intron junction region within spliced region
        
        2. splice-site-slip, pseudo-exon-inclusion
           in addition to exon-intron junction region, 
           we check for the region around non exon-intron junction break points 
        Maybe, we should check the exon-intron junction region within, 
        e.g., +-10000bp from the spliced region to search for unknown phenomena.
    """

    donor_size_exon, donor_size_intron = [int(x) for x in donor_size.split(',')]
    acceptor_size_intron, acceptor_size_exon = [int(x) for x in acceptor_size.split(',')]
    branch_size_intron, branch_size_exon = [int(x) for x in branch_size.split(',')]

    searchMargin1 = 100 
    searchMargin2 = 10

    splicingDonnorMotif = ["AG", "GTRAGT"]
    splicingAcceptorMotif = ["YYYYNCAG", "G"]

    annot_utils.exon.make_exon_info(output_file + ".tmp.refExon.bed.gz", "refseq", genome_id, is_grc, True)

    hout = open(output_file, 'w')

    mutation_tb = pysam.TabixFile(mutation_file)
    exon_tb = pysam.TabixFile(output_file + ".tmp.refExon.bed.gz")
    if is_branchpoint:
        if genome_id != "hg19":
            print("branchpoint can be used only for hg19", file = sys.stderr)
            sys.exit(1)
        if is_grc:
            branchpoint_bed = pkg_resources.resource_filename("annot_utils", "data/hg19/branchpoint_signal.grc.bed.gz")
        else:
            branchpoint_bed = pkg_resources.resource_filename("annot_utils", "data/hg19/branchpoint_signal.bed.gz")
        branch_tb = pysam.TabixFile(branchpoint_bed)

    header2ind = {}
    with open(input_file, 'r') as hin:
        
        # read header
        header = hin.readline().rstrip('\n').split('\t')
        for i in range(len(header)):
            header2ind[header[i]] = i

        # print header
        print('\t'.join(header) + '\t' + "Mutation_Key" + '\t' + "Motif_Pos" + '\t' + "Mutation_Type" + '\t' + "Is_Canonical", file = hout)

        for line in hin:
            F = line.rstrip('\n').split('\t') 

            sj_start = int(F[header2ind["SJ_2"]]) - 1
            sj_end = int(F[header2ind["SJ_3"]]) + 1

            if F[header2ind["Splicing_Class"]] not in ["Exon skipping", "Alternative 3'SS", "Alternative 5'SS",
                                                       "Intronic alternative 3'SS", "Intronic alternative 5'SS"]: continue

            firstSearchRegion = [F[header2ind["SJ_1"]], sj_start, sj_end]
            splicingMotifRegions = []
            targetGene  =[]

            # we need to detect the non exon-intron junction break points
            # current procedure may be not perfect and be subject to change..

            gene1 = F[header2ind["Gene_1"]].split(';')
            gene2 = F[header2ind["Gene_2"]].split(';')
            junction1 = F[header2ind["Is_Boundary_1"]].split(';')   
            junction2 = F[header2ind["Is_Boundary_2"]].split(';')
            offset1 = F[header2ind["Offset_1"]].split(';')
            offset2 = F[header2ind["Offset_2"]].split(';')

            # just consider genes sharing the exon-intron junction with the breakpoints of splicings
            for i in range(0, len(gene1)):
                if junction1[i] != "*": targetGene.append(gene1[i])
            for i in range(0, len(gene2)):
                if junction2[i] != "*": targetGene.append(gene2[i])
            targetGene = list(set(targetGene))


            if F[header2ind["Splicing_Class"]] in ["Alternative 3'SS", "Alternative 5'SS",
                                                   "Intronic alternative 3'SS", "Intronic alternative 5'SS"]:

                # for non exon-intron junction breakpoints
                for i in range(0, len(gene1)):

                    if junction1[i] == "*" and junction2[i] == "s": # splicing donor motif, plus direction
                        firstSearchRegion[1] = sj_start - searchMargin1
                        motif_start = sj_start - donor_size_exon + 1 - int(offset2[i])
                        motif_end = sj_start + donor_size_intron - int(offset2[i])
                        splicingMotifRegions.append((F[header2ind["SJ_1"]], motif_start, motif_end, "Donor", "+", 0))
                    if junction1[i] == "*" and junction2[i] == "e": # splicing acceptor motif, minus direction
                        firstSearchRegion[1] = sj_start - searchMargin1 
                        motif_start = sj_start - acceptor_size_exon + 1 - int(offset2[i])
                        motif_end = sj_start + acceptor_size_intron - int(offset2[i])
                        splicingMotifRegions.append((F[header2ind["SJ_1"]], motif_start, motif_end, "Acceptor", "-", 0))
                    if junction1[i] == "s" and junction2[i] == "*": # splicing donor motif, minus direction
                        firstSearchRegion[2] = sj_end + searchMargin1
                        motif_start = sj_end - donor_size_intron - int(offset1[i])
                        motif_end = sj_end + donor_size_exon - 1 - int(offset1[i])
                        splicingMotifRegions.append((F[header2ind["SJ_1"]], motif_start, motif_end, "Donor", "-", 0))
                    if junction1[i] == "e" and junction2[i] == "*": # # splicing acceptor motif, plus direction
                        firstSearchRegion[2] = sj_end + searchMargin1
                        motif_start = sj_end - acceptor_size_intron - int(offset1[i])
                        motif_end = sj_end + acceptor_size_exon - 1 - int(offset1[i])
                        splicingMotifRegions.append((F[header2ind["SJ_1"]], motif_start, motif_end, "Acceptor", "+", 0))


            ##########
            # rough check for the mutation between the spliced region
            tabixErrorFlag1 = 0
            try:
                mutation_lines = mutation_tb.fetch(firstSearchRegion[0], firstSearchRegion[1], firstSearchRegion[2])
            except Exception as inst:
                # print >> sys.stderr, "%s: %s at the following key:" % (type(inst), inst.args)
                # print >> sys.stderr, '\t'.join(F)
                tabixErrorFlag1 = 1

            # if there are some mutaions
            if tabixErrorFlag1 == 0 and mutation_lines is not None:

                # chr_name = grch2ucsc[F[header2ind["SJ_1"]]] if F[header2ind["SJ_1"]] in grch2ucsc else F[header2ind["SJ_1"]] 
                chr_name = F[header2ind["SJ_1"]] 
                # check the exons within the spliced regions
                tabixErrorFlag2 = 0
                try:
                    exon_lines = exon_tb.fetch(chr_name, firstSearchRegion[1], firstSearchRegion[2])
                except Exception as inst:
                    # print >> sys.stderr, "%s: %s at the following key:" % (type(inst), inst.args)
                    # print >> sys.stderr, '\t'.join(F)
                    tabixErrorFlag2 = 1

                # first, add the exon-intron junction for detailed check region list
                if tabixErrorFlag2 == 0:
                    for exon_line in exon_lines:
                        exon = exon_line.split('\t')
                        if exon[3] not in targetGene: continue
                        if exon[5] == "+":
                            # splicing acceptor motif, plus direction
                            splicingMotifRegions.append((exon[0], int(exon[1]) - acceptor_size_intron + 1, int(exon[1]) + acceptor_size_exon, "Acceptor", "+", 1))
                            # splicing donor motif, plus direction
                            splicingMotifRegions.append((exon[0], int(exon[2]) - donor_size_exon + 1, int(exon[2]) + donor_size_intron, "Donor", "+", 1))
                        if exon[5] == "-":
                            # splicing donor motif, minus direction 
                            splicingMotifRegions.append((exon[0], int(exon[1]) - donor_size_intron + 1, int(exon[1]) + donor_size_exon, "Donor", "-", 1))
                            # splicing acceptor motif, minus direction
                            splicingMotifRegions.append((exon[0], int(exon[2]) - acceptor_size_exon + 1, int(exon[2]) + acceptor_size_intron, "Acceptor", "-", 1))


                # check branchpoint within the spliced regions
                if is_branchpoint:
                    tabixErrorFlag3 = 0
                    try:
                        branche_lines = branch_tb.fetch(chr_name, firstSearchRegion[1], firstSearchRegion[2])
                    except Exception as inst:
                        print("%s: %s at the following key:" % (type(inst), inst.args), file = sys.stderr)
                        print('\t'.join(F), file = sys.stderr)
                        tabixErrorFlag3 = 3

                    if tabixErrorFlag3 == 0:
                        for branch_line in branche_lines:
                            branch = branch_line.split('\t')
                            if branch[5] == "+":
                                splicingMotifRegions.append((branch[0], int(branch[2]) - branch_size_intron, int(branch[2]) + branch_size_exon - 1, "Branchpoint", "+", 1))
                            else:
                                splicingMotifRegions.append((branch[0], int(branch[2]) - branch_size_exon + 1, int(branch[2]) + branch_size_intron, "Branchpoint", "-", 1))


                splicingMotifRegions = list(set(splicingMotifRegions))

                # compare each mutation with exon-intron junction regions and non-exon-intorn junction breakpoints.
                for mutation_line in mutation_lines:
                    mutation = mutation_line.split('\t')
                    RegMut = []
                    for reg in splicingMotifRegions:

                        # insertion or deletion (just consider the disruption of splicing motifs now)
                        # if (len(mutation[3]) > 1 or len(mutation[4]) > 1) and reg[5] == 1:
                        if (len(mutation[3]) > 1 or len(mutation[4]) > 1):

                            indel_start = int(mutation[1]) + 1
                            indel_end = int(mutation[1]) + len(mutation[3]) - 1 if len(mutation[3]) > 1 else indel_start
                            if indel_start <= reg[2] and reg[1] <= indel_end:
                           
                                if reg[3] in ["Acceptor", "Donor"]: 
                                    if reg[3] == "Acceptor" and reg[4] == "+": canonical_start_pos = reg[2] - acceptor_size_exon - 1
                                    if reg[3] == "Acceptor" and reg[4] == "-": canonical_start_pos = reg[1] + acceptor_size_exon 
                                    if reg[3] == "Donor" and reg[4] == "+": canonical_start_pos = reg[1] + donor_size_exon 
                                    if reg[3] == "Donor" and reg[4] == "-": canonical_start_pos = reg[2] - donor_size_exon - 1

                                    is_canonical = "Non-canonical"
                                    if len(mutation[3]) > 1: # deletion
                                        if indel_start <= canonical_start_pos + 1 and canonical_start_pos <= indel_end:
                                            is_canonical = "Canonical"
                                    else: # insertion
                                        if indel_start == canonical_start_pos + 1:
                                            is_canonical = "Canonical"

                                # branch point
                                else:
                                    if reg[4] == "+": canonical_start_pos = reg[2] - branch_size_exon + 1
                                    if reg[4] == "-": canonical_start_pos = reg[1] + branch_size_exon - 1
 
                                    is_canonical = "Non-canonical"
                                    if len(mutation[3]) > 1: # deletion
                                        if indel_start <= canonical_start_pos and canonical_start_pos <= indel_end:
                                            is_canonical = "Canonical"
                                    else: # insertion
                                        pass





                                if reg[5] == 0: RegMut.append([reg, reg[3] + " creation", is_canonical])
                                if reg[5] == 1: RegMut.append([reg, reg[3] + " disruption", is_canonical])
 
                        # base substitution
                        if len(mutation[3]) == 1 and len(mutation[4]) == 1 and reg[1] <= int(mutation[1]) <= reg[2]:
                            motifSeq = ""
                            if reg[3] == "Acceptor": motifSeq = splicingAcceptorMotif[0] + splicingAcceptorMotif[1]                  
                            if reg[3] == "Donor": motifSeq = splicingDonnorMotif[0] + splicingDonnorMotif[1]

                            is_canonical = "Non-canonical"
                            if reg[3] in ["Acceptor", "Donor"]:
                                if reg[3] == "Acceptor" and reg[4] == "+": canonical_start_pos = reg[2] - acceptor_size_exon - 1
                                if reg[3] == "Acceptor" and reg[4] == "-": canonical_start_pos = reg[1] + acceptor_size_exon 
                                if reg[3] == "Donor" and reg[4] == "+": canonical_start_pos = reg[1] + donor_size_exon 
                                if reg[3] == "Donor" and reg[4] == "-": canonical_start_pos = reg[2] - donor_size_exon - 1

                                is_canonical = "Canonical" if canonical_start_pos <= int(mutation[1]) <= canonical_start_pos + 1 else "Non-canonical"

                            # branchpoint 
                            else:
                                if reg[4] == "+": canonical_start_pos = reg[2] - branch_size_exon + 1 
                                if reg[4] == "-": canonical_start_pos = reg[1] + branch_size_exon - 1 

                                is_canonical = "Canonical" if canonical_start_pos == int(mutation[1]) else "Non-canonical"


                            if reg[5] == 0: RegMut.append([reg, reg[3] + " creation", is_canonical])
                            if reg[5] == 1: RegMut.append([reg, reg[3] + " disruption", is_canonical])

                    for item in sorted(RegMut):
                        mut_print_str = ','.join([mutation[i] for i in [0, 1, 3, 4]])
                        print('\t'.join(F) + '\t' + mut_print_str + '\t' + F[header2ind["SJ_1"]] + ':' + str(item[0][1]) + '-' + str(item[0][2]) + ',' + item[0][4] + '\t' + item[1] + '\t' + item[2], file = hout)


    hout.close()

    subprocess.call(["rm", "-rf", output_file + ".tmp.refExon.bed.gz"])
    subprocess.call(["rm", "-rf", output_file + ".tmp.refExon.bed.gz.tbi"])



def get_snv_junction_only_dist(input_file, output_file, mutation_file, annotation_dir, searchMargin):

    ref_exon_bed = annotation_dir + "/refExon.bed.gz"
    grch2ucsc_file = annotation_dir + "/grch2ucsc.txt"

    # relationship between CRCh and UCSC chromosome names
    grch2ucsc = {}
    with open(grch2ucsc_file, 'r') as hin:
        for line in hin:
            F = line.rstrip('\n').split('\t')
            grch2ucsc[F[0]] = F[1]

    hout = open(output_file, 'w')
    mutation_tb = tabix.open(mutation_file)

    header2ind = {}
    with open(input_file, 'r') as hin:
        
        # read header
        header = hin.readline().rstrip('\n').split('\t')
        for i in range(len(header)):
            header2ind[header[i]] = i

        # print header
        print('\t'.join(header) + '\t' + "Mutation_Key" + '\t' + "Motif_Pos" + '\t' + "Mutation_Type" + '\t' + "Is_Canonical", file = hout)

        for line in hin:
            F = line.rstrip('\n').split('\t') 

            sj_start = int(F[header2ind["SJ_2"]]) - 1
            sj_end = int(F[header2ind["SJ_3"]]) + 1

            if F[header2ind["Splicing_Class"]] not in ["Exon skipping", "Alternative 3'SS", "Alternative 5'SS",
                                                       "Intronic alternative 3'SS", "Intronic alternative 5'SS"]: continue


            firstSearchRegion = [F[header2ind["SJ_1"]], sj_start, sj_end]

            ##########
            # rough check for the mutation between the spliced region
            tabixErrorFlag1 = 0
            try:
                mutations = mutation_tb.query(firstSearchRegion[0], firstSearchRegion[1] - searchMargin, firstSearchRegion[2] + searchMargin)
            except Exception as inst:
                # print >> sys.stderr, "%s: %s at the following key:" % (type(inst), inst.args)
                # print >> sys.stderr, '\t'.join(F)
                tabixErrorFlag1 = 1

            # if there are some mutaions
            if tabixErrorFlag1 == 0 and mutations is not None:

                for mutation in mutations:

                    mut_print_str = ','.join([mutation[i] for i in [0, 1, 3, 4]])
                    print('\t'.join(F) + '\t' + mut_print_str + '\t' + "---,---" + '\t' + "---" + '\t' + "---", file = hout)


    hout.close()

def get_sv_junction(input_file, output_file, sv_file, genome_id, is_grc):
# def get_sv_junction(input_file, output_file, mutation_file, annotation_dir):

    from collections import namedtuple
    SV = namedtuple('SV', ('chr1', 'pos1', 'dir1', 'chr2', 'pos2', 'dir2', 'inseq', 'sv_type'))

    """
        a script for detecting candidate structural variations (currently just deletions causing splicing changes)
    """

    sv_comp_margin = 20
    exon_comp_margin = 10


    hout = open(output_file, 'w')

    sv_tb = pysam.TabixFile(sv_file)
    # annot_utils.exon.make_exon_info(output_file + ".tmp.refExon.bed.gz", "refseq", genome_id, is_grc, True)
    # exon_tb = pysam.TabixFile(output_file + ".tmp.refExon.bed.gz")

    header2ind = {}
    with open(input_file, 'r') as hin:

        # read header
        header = hin.readline().rstrip('\n').split('\t')
        for i in range(len(header)):
            header2ind[header[i]] = i

        # print header
        print('\t'.join(header) + '\t' + "SV_Key" + '\t' + "SV_Type" + '\t' + "Dist_To_Junc1" + '\t' + "Dist_To_Junc2", file = hout)


        for line in hin:
            F = line.rstrip('\n').split('\t')

            sj_chr = F[header2ind["SJ_1"]]
            sj_start = int(F[header2ind["SJ_2"]]) - 1
            sj_end = int(F[header2ind["SJ_3"]]) + 1

            gene1 = F[header2ind["Gene_1"]].split(';')
            gene2 = F[header2ind["Gene_2"]].split(';')
            junction1 = F[header2ind["Is_Boundary_1"]].split(';')
            junction2 = F[header2ind["Is_Boundary_2"]].split(';')


            """
            # just consider exon skipping genes
            for i in range(0, len(gene1)):
                if junction1[i] != "*" and junction2[i] != "*": 
                    targetGene.append(gene1[i])
                    targetGene.append(gene2[i])
            targetGene = list(set(targetGene))
            """

            associated_sv_list = []
            ##########
            # rough check for the mutation between the spliced region
            tabixErrorFlag1 = 0
            try:
                sv_list = sv_tb.fetch(sj_chr, sj_start - sv_comp_margin, sj_end + sv_comp_margin)
            except ValueError as inst:
                # print >> sys.stderr, "%s: %s at the following key:" % (type(inst), inst.args)
                tabixErrorFlag1 = 1

            # if there are some mutaions
            if tabixErrorFlag1 == 0 and sv_list is not None:
        
                for sv_cur in sv_list:

                    sv_curs = sv_cur.split('\t')
                    SV_cur = SV(chr1 = sv_curs[3], pos1 = int(sv_curs[4]), dir1 = sv_curs[5], chr2 = sv_curs[6], pos2 = int(sv_curs[7]), dir2 = sv_curs[8], inseq = sv_curs[9], sv_type = sv_curs[10])
                    if SV_cur.sv_type != "deletion": continue # For now, we just consider deletion
            

                    # the SV should be deletion confied within spliced junction
                    if  SV_cur.chr1 == sj_chr and SV_cur.chr2 == sj_chr and \
                      sj_start - sv_comp_margin <= SV_cur.pos1 and SV_cur.pos2 <= sj_end + sv_comp_margin:

                        """ 
                        # the splicing junction should be shared by SV breakpoint or exon-intron junction
                        junc_flag1 = 0
                        for i in range(0, len(gene1)):
                            if junction1[i] != "*": junc_flag1 = 1
                        
                        junc_flag2 = 0
                        for i in range(0, len(gene2)): 
                            if junction2[i] != "*": junc_flag2 = 1 
                          
                        if junc_flag1 == 1 or abs(sj_start - int(mutation[2])) <= sv_comp_margin and \
                          junc_flag2 == 1 or abs(sj_end - int(mutation[5])) <= sv_comp_margin:
                            mutation_sv.append('\t'.join(mutation))
                        """

                        # if F[header2ind["Splicing_Class"]] in ["exon-skip", "splice-site-slip", "pseudo-exon-inclusion"]: 
                        if F[header2ind["Splicing_Class"]] in ["Exon skipping", "Alternative 3'SS", "Alternative 5'SS",
                                                               "Intronic alternative 3'SS", "Intronic alternative 5'SS"]:


                            associated_sv_list.append(SV_cur)
                        elif abs(sj_start - SV_cur.pos1) <= sv_comp_margin and abs(sj_end - SV_cur.pos2) <= sv_comp_margin:
                            associated_sv_list.append(SV_cur)

                            

            for SV_cur in associated_sv_list:

                junc_to_dist1 = SV_cur.pos1 - sj_start
                junc_to_dist2 = sj_end - SV_cur.pos2

                sv_key = ','.join([SV_cur.chr1, str(SV_cur.pos1), SV_cur.dir1, SV_cur.chr2, str(SV_cur.pos2), SV_cur.dir2, SV_cur.inseq])

                print('\t'.join(F) + '\t' + sv_key + '\t' + SV_cur.sv_type + '\t' + str(junc_to_dist1) + '\t' + str(junc_to_dist2), file = hout)

    hout.close()




#! /usr/bin/env python

from __future__ import print_function
import sys, re, subprocess
import annot_utils.gene, annot_utils.exon
import pysam

def check_splicing_junction_for_exonization(exonizaiton_junction, output_file, genome_id, is_grc, min_new_intron_size, boundary_margin):

    annot_utils.gene.make_gene_info(output_file + ".tmp.refGene.bed.gz", "refseq", genome_id, is_grc, True)
    annot_utils.exon.make_exon_info(output_file + ".tmp.refExon.bed.gz", "refseq", genome_id, is_grc, True)

    gene_tb = pysam.TabixFile(output_file + ".tmp.refGene.bed.gz")
    exon_tb = pysam.TabixFile(output_file + ".tmp.refExon.bed.gz")

    pos_match = re.match(r'([\w\d]+)\:(\d+)\-(\d+)', exonizaiton_junction)
    if pos_match is None:
        print("exonization_junction is not the right format", file = sys.stderr)
        sys.exit(1)

    schr, sstart, send = pos_match.group(1), int(pos_match.group(2)), int(pos_match.group(3))


    ##########
    is_same_gene, is_boundary, is_intron = False, False, False
    new_exon_info = []
    # check gene annotation for the side 1  
    tabixErrorFlag = 0
    try:
        records = gene_tb.fetch(schr, sstart, send)
    except Exception as inst:
        # print >> sys.stderr, "%s: %s at the following key:" % (type(inst), inst.args)
        # print >> sys.stderr, '\t'.join(F)
        tabixErrorFlag = 1

    if tabixErrorFlag == 0:
        for record_line in records:
            record = record_line.split('\t')
            if send > int(record[1]) and send < int(record[2]):
                is_same_gene = True
                tgene_chr, tgene_start, tgene_end, tgene, tstrand = record[0], int(record[1]), int(record[2]), record[3], record[5]

                # check exon information
                tabixErrorFlag2 = 0
                try:
                    records2 = exon_tb.fetch(tgene_chr, tgene_start, tgene_end)
                except Exception as inst:
                    tabixErrorFlag2 = 1

                tstarts, tends = [], []
                if tabixErrorFlag2 == 0:
                    for record_line2 in records2:
                        record2 = record_line2.split('\t')
                        if record2[3] == tgene and record2[5] == tstrand:
                            tstarts.append(int(record2[1]) + 1) # the last exonic positions
                            tends.append(int(record2[2])) # the first exonic positions

                    tstarts.sort()
                    tends.sort()

                    for i in range(len(tends) - 1):
                        if abs((sstart - 1) - tends[i]) <= boundary_margin and tstarts[i + 1] - (send + 1) - 1 >= min_new_intron_size:
                            if tstrand == '+':
                                new_exon_info.append((schr, tends[i], tstarts[i + 1], send + 1, '+', "acceptor"))
                            else:
                                new_exon_info.append((schr, tends[i], tstarts[i + 1], send + 1, '-', "donor"))

                        if abs((send + 1) - tstarts[i + 1]) <= boundary_margin and (sstart - 1) - tends[i] - 1 >= min_new_intron_size:
                            if tstrand == '+':
                                new_exon_info.append((schr, tstarts[i + 1], tends[i], sstart - 1, '+', "donor"))
                            else:
                                new_exon_info.append((schr, tstarts[i + 1], tends[i], sstart - 1, '-', "acceptor"))


    new_exon_info = list(set(new_exon_info))

    subprocess.call(["rm","-rf", output_file + ".tmp.refGene.bed.gz"])
    subprocess.call(["rm","-rf", output_file + ".tmp.refGene.bed.gz.tbi"])
    subprocess.call(["rm","-rf", output_file + ".tmp.refExon.bed.gz"])
    subprocess.call(["rm","-rf", output_file + ".tmp.refExon.bed.gz.tbi"])

    return new_exon_info



def check_opposite_junction(junc_file, exonization_info, output_file, control_file, read_num_thres, max_new_exon_size, boundary_margin):

    if control_file is not None:
        control_db = pysam.TabixFile(control_file)

    hout = open(output_file, 'w')

    with open(junc_file, 'r') as hin:
        for line in hin:
            F = line.rstrip('\n').split('\t')
            if F[0] != exonization_info[0][0]: continue

            new_exon_flag = 0
            for i in range(len(exonization_info)):
                if (exonization_info[i][4] == '+' and exonization_info[i][5] == "acceptor") or \
                   (exonization_info[i][4] == '-' and exonization_info[i][5] == "donor"):
                    if int(F[1]) - 1 >= exonization_info[i][3] and int(F[1]) - 1 <= exonization_info[i][3] + max_new_exon_size and \
                       abs(int(F[2]) + 1 - exonization_info[i][2]) <= boundary_margin and int(F[6]) >= read_num_thres:
                        new_exon_flag = 1
                else:
                    if int(F[2]) +  1 >= exonization_info[i][3] - max_new_exon_size and int(F[2]) + 1 <= exonization_info[i][3] and \
                        abs(int(F[1]) - 1 - exonization_info[i][2]) <= boundary_margin and int(F[6]) >= read_num_thres:
                        new_exon_flag = 1

            if new_exon_flag == 1:

                control_flag = 0
                if control_file is not None:
                    tabixErrorFlag = 0
                    try:
                        records = control_db.fetch(F[0], int(F[1]) - 5, int(F[1]) + 5)
                    except Exception as inst:
                        # prin >> sys.stderr, "%s: %s" % (type(inst), inst.args)
                        # tabixErrorMsg = str(inst.args)
                        tabixErrorFlag = 1

                    if tabixErrorFlag == 0:
                        for record_line in records:
                            record = record_line.split('\t')
                            if F[0] == record[0] and F[1] == record[1] and F[2] == record[2]:
                                control_flag = 1

                if control_flag == 0:
                    print('\t'.join(F), file = hout)

     
    hout.close()




